# https://setuptools.pypa.io/en/latest/userguide/pyproject_config.html
# If compatibility with legacy builds or versions of tools that don’t support
# certain packaging standards (e.g. PEP 517 or PEP 660), a simple setup.py
# script can be added to your project [1] (while keeping the configuration in
# pyproject.toml):

from setuptools import setup

setup()
